package checkers.igj;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import checkers.types.*;

/**
 * Provides some state for the {@code IGJVisitor}.
 * 
 * So far, it stores two objects: 1. The immutability of the scope the
 * visitor is in 2. Binding for resolving wildcards
 */
public class VisitorState {
    public IGJImmutability thisImmutability = IGJImmutability.READONLY;

    /** a Map represnting the bindings * */
    private Map<AnnotatedClassType, Map<String, AnnotationData>> bindings =
        new HashMap<AnnotatedClassType, Map<String, AnnotationData>>();

    /**
     * 
     * 
     * @param type
     * @param data
     * @return
     */
    public boolean bind(AnnotatedClassType type, String id,
            AnnotationData data) {

        if (!bindings.containsKey(type))
            bindings.put(type, new HashMap<String, AnnotationData>());

        return bindings.get(type).put(id, data) == null;
    }

    public boolean unbind(AnnotatedClassType type) {
        return (bindings.remove(type) != null);
    }

    public Map<String, AnnotationData> getBindings(AnnotatedClassType type) {
        if (bindings.get(type) == null)
            return Collections.<String, AnnotationData> emptyMap();
        else
            return Collections.unmodifiableMap(bindings.get(type));
    }
}
