package checkers.nonnull;

import checkers.source.*;
import checkers.subtype.*;
import checkers.quals.*;
import checkers.types.*;

import com.sun.source.tree.*;

/**
 * A type-checking visitor for the {@code @NonNull} type qualifier
 * that uses the {@link SubtypeVisitor} implementation.
 *
 * @see SubtypeVisitor
 */
public class NonnullVisitor extends SubtypeVisitor {

    /**
     * Creates a new visitor for type-checking {@code @NonNull}.
     *
     * @param checker the {@link SubtypeChecker} to use
     * @param root the root of the input program's AST to check
     */
    public NonnullVisitor(SubtypeChecker checker, CompilationUnitTree root) {
        super(checker, root);
    }

    @Override
    public Void visitUnary(UnaryTree node, Void p) {

        if (!(node.getKind() == Tree.Kind.POSTFIX_DECREMENT ||
                node.getKind() == Tree.Kind.POSTFIX_INCREMENT ||
                node.getKind() == Tree.Kind.PREFIX_DECREMENT ||
                node.getKind() == Tree.Kind.PREFIX_INCREMENT))
            return super.visitUnary(node, p);

        AnnotatedClassType expression = factory.getClass(node.getExpression());

        if (!expression.hasAnnotationAt(NonNull.class, AnnotationLocation.RAW))
            checker.report(Result.failure("unary.invalid"), node.getExpression());

        return super.visitUnary(node, p);
    }
}
