package tests;

import java.io.StringWriter;
import java.util.*;
import javax.annotation.processing.*;
import javax.tools.*;
import java.io.*;

public class TestInput {

    private JavaCompiler compiler;

    private StandardJavaFileManager fileManager;

    private Iterable<? extends JavaFileObject> files;
    private Iterable<String> processors;
    private List<String> options;

    public TestInput(Iterable<? extends JavaFileObject> files,
                     Iterable<String> processors, String[] options) {

        this.compiler = ToolProvider.getSystemJavaCompiler();
        this.fileManager = compiler.getStandardFileManager(null, null, null);

        this.files = files;
        this.processors = processors;
        this.options = new LinkedList<String>();

        String outputDir = System.getProperty("tests.outputDir", "build/testclasses");
        String classpath = System.getProperty("tests.classpath", "build");

        this.options.add("-g");
        this.options.add("-d");
        this.options.add(outputDir);
        this.options.add("-classpath");
        this.options.add("../checkers.jar:junit.jar:"+classpath);
        this.options.addAll(Arrays.asList(options));
    }

    public TestRun run() {
        StringWriter output = new StringWriter();
        DiagnosticCollector<JavaFileObject> diagnostics = new
            DiagnosticCollector<JavaFileObject>();

        JavaCompiler.CompilationTask task = compiler.getTask(output, fileManager,
              diagnostics, this.options, this.processors, this.files);

        Boolean result = task.call();

        return new TestRun(result, output.toString(), diagnostics.getDiagnostics());
    }

    public static TestRun compileAndCheck(Iterable<? extends JavaFileObject> files,
                                          String processor) {
        TestInput input = new TestInput(files,
                Collections.<String>emptySet(),
                new String[] { "-typeprocessor", processor });
        return input.run();
    }
}
